#! /usr/bin/env node

const readline = require("readline");
const fs = require('fs');
const path = require('path');
const localPath = path.join(process.cwd(), 'node_modules', 'nuance-gulp-build');
const common = require("../lib/common")

installers = {
    "default" : () => require("../lib/installers/default")
}

function checkLocalInstallExists(){
    var exists = fs.existsSync(localPath);
    if(!exists){
        console.error("local installation does not exists. Please install locally");
        process.exit(1);
    }
}

init = async function (args) {
    var installer = installers['default']().run({localPath: localPath})
    const rl = readline.createInterface({ input: process.stdin, output: process.stdout });
    
    config = await common.config.load(path.join(process.cwd(), ".buildconfig.json"));
    try{
        config.build_env = await common.question(rl, "which environment this build will run? (gitlab|bitbucket) ");
    }finally{
        rl.close();
    }
    
    common.config.save(path.join(process.cwd(), ".buildconfig.json"), config);
    
    console.log("install complete. Please commit it on your repository");
    rl.close();
};
async function envSet(k, v){
    var cfg = await common.config.load(path.join(process.cwd(), 'buildinfo.json'));
    cfg[k] = v;
    await common.config.save(path.join(process.cwd(), 'buildinfo.json'), cfg);
}


const yargs = require("yargs")

.command('init', 'Initializes the build agent', function( argv ) { 
    checkLocalInstallExists();
    init(argv).then(r=>{}).catch((ex)=>{ 
        console.error("an error occurred in initialization:", ex)

    }); 
})
.command('project <creator> <cmd>', 'adds a .net project configuration', {}, function( argv ) { 
    checkLocalInstallExists();
    var creator = require(`../lib/installers/${argv.creator}`);
    var cmd = creator[argv.cmd];
    cmd(argv).then(()=>{});
    return;
})
.command("env set <name> <value>", "adds an environment to debug as local code", {}, function(argv){
    checkLocalInstallExists();
    envSet(argv.name, argv.value).catch((ex)=>{
        console.log(ex);
    })
})
.argv;